@@ -2,6 +2,7 @@
 #' @title syntaxProcessing
 #' @description extracts performance metrics from cross-validation
 #' @importFrom memuse Sys.meminfo
+#' @importFrom tools file_ext
 #' @author E. F. Haghish
 #' @keywords Internal
 #' @noRd
@@ -9,8 +10,8 @@
 
 syntaxProcessing <- function(data, preimpute, impute, ram,
                              matching, miniter, maxiter, max_models,
-                             tuning_time, cv, weights_column,
-                             verbosity, report) {
+                             tuning_time, cv,
+                             verbosity, report, save) {
 
   #if ("GBM" %in% impute) {
     #if (nrow(data) < 200) stop("too few rows... use 'ELNET' or 'DRF' instead")
@@ -29,18 +30,19 @@ syntaxProcessing <- function(data, preimpute, impute, ram,
     #length(formula <- as.character(formula)) == 3L,
     #"'max_models' must be a positive integer equal or more than 1" = max_models >= 1,
     #"'tuning_time' must be a positive integer" = tuning_time >= 2,
-    "'cv' must be a positive integer equal or more than 10" = cv >= 10
+    "'cv' must be a positive integer equal or more than 10" = cv >= 10,
+    "'save' argument must have '*.mlim' file extension" = tools::file_ext(save) == "mlim"
   )
 
   #if (miniter < 2 & preimpute == "iterate") stop("'miniter' must not be less than 2")
   #if ( maxiter < 2 & preimpute == "iterate") stop("'maxiter' must not be less than 2")
 
-  if (!is.null(weights_column)) {
-    stopifnot(
-      "'weights_column' must have equal length with the data.frame" = length(weights_column) == nrow(data),
-      "'weights_column' must not have any missing observations" = !anyNA(weights_column)
-    )
-  }
+  # if (!is.null(weights_column)) {
+  #   stopifnot(
+  #     "'weights_column' must have equal length with the data.frame" = length(weights_column) == nrow(data),
+  #     "'weights_column' must not have any missing observations" = !anyNA(weights_column)
+  #   )
+  # }
 
 
   if (!is.null(ram)) {
